# vidsrc/__init__.pyi

"""Video Frameserver for Numpy."""

from collections.abc import Sequence
from typing import Any, overload

from numpy.typing import NDArray

__version__: str


class VideoSource(Sequence[NDArray[Any]]):
    """Access frames of video file as numpy arrays.

    Parameters:
        filename:
            Name of file to open.
        framerate:
            Video frame rate in fps.
        grayscale:
            Return frames as normalized gray scale.

    """

    filename: str
    """Name of open file."""

    duration: float
    """Video duration in s."""

    framerate: float
    """Video frame rate in fps."""

    shape: tuple[int, ...]
    """Shape of video data.

    Number of frames, frame height, frame width, color channels.
    """

    def __init__(
        self,
        filename: str,
        /,
        *,
        framerate: float = 0.0,
        grayscale: bool = False,
    ) -> None: ...

    @overload
    def __getitem__(self, key: int, /) -> NDArray[Any]: ...

    @overload
    def __getitem__(
        self, key: slice[Any, Any, Any], /
    ) -> Sequence[NDArray[Any]]: ...

    def __len__(self) -> int: ...



# test_vidsrc.py

import numpy
import pytest
import vidsrc


def test_module() -> None:
    """Test vidsrc module."""

    assert vidsrc.__version__ in vidsrc.__doc__


def test_videosource() -> None:
    """Test VideoSource class."""
    video = vidsrc.VideoSource('test.avi')
    assert video.filename == 'test.avi'
    assert len(video) == 48
    assert pytest.approx(video.framerate, abs=1e-2) == 29.97
    assert pytest.approx(video.duration, abs=1e-2) == 1.6016
    assert video.shape == (48, 64, 64, 3)
    assert video[0].dtype == numpy.uint8
    assert video[0].shape == (64, 64, 3)
    mean = numpy.asarray([frame.mean() for frame in video])
    numpy.testing.assert_allclose(
        mean[[1, -1]], [6.871582, 18.197266], atol=1e-3
    )


def test_videosource_options() -> None:
    """Test VideoSource class with options."""
    video = vidsrc.VideoSource('test.avi', framerate=30.0, grayscale=True)
    assert video.filename == 'test.avi'
    assert len(video) == 48
    assert pytest.approx(video.framerate, abs=1e-2) == 30.0
    assert pytest.approx(video.duration, abs=1e-2) == 1.6016
    assert video.shape == (48, 64, 64)
    assert video[0].dtype == numpy.float64
    assert video[0].shape == (64, 64)
    mean = numpy.asarray([frame.mean() for frame in video])
    numpy.testing.assert_allclose(
        mean[[1, -1]], [0.026947, 0.071362], atol=1e-3
    )



# vidsrc/setup.py

"""Vidsrc package Setuptools script."""

import re
import sys

import numpy
from setuptools import Extension, setup

buildnumber = ''


def search(pattern: str, string: str, flags: int = 0) -> str:
    """Return first match of pattern in string."""
    match = re.search(pattern, string, flags)
    if match is None:
        raise ValueError(f'{pattern!r} not found')
    return match.groups()[0]


def fix_docstring_examples(docstring: str) -> str:
    """Return docstring with examples fixed for GitHub."""
    start = True
    indent = False
    lines = ['..', '  This file is generated by setup.py', '']
    for line in docstring.splitlines():
        if not line.strip():
            start = True
            indent = False
        if line.startswith('>>> '):
            indent = True
            if start:
                lines.extend(['.. code-block:: python', ''])
                start = False
        lines.append(('    ' if indent else '') + line)
    return '\n'.join(lines)


with open('vidsrc/vidsrc.cpp', encoding='utf-8') as fh:
    code = fh.read()

version = search(r'#define _VERSION_ "(.*?)"', code).replace('.x.x', '.dev0')
version += ('.' + buildnumber) if buildnumber else ''

readme = search(
    r'(?:\r\n|\r|\n)\"(.*)\"(?:\r\n|\r|\n){2}\#define _VERSION_',
    code,
    re.MULTILINE | re.DOTALL,
).replace('\\n\\', '')

description = readme.splitlines()[0][:-1]

readme = '\n'.join(
    [description, '=' * len(description)] + readme.splitlines()[1:]
)

if 'sdist' in sys.argv:

    with open('README.rst', 'w', encoding='utf-8') as fh:
        fh.write(fix_docstring_examples(readme))

    license = search(
        r'(Copyright.*)\*/(?:\r\n|\r|\n){2}\#define _DOC_',
        code,
        re.MULTILINE | re.DOTALL,
    )
    license = license.replace('# ', '').replace('#', '')

    with open('LICENSE', 'w', encoding='utf-8') as fh:
        fh.write('BSD-3-Clause license\n\n')
        fh.write(license)

DIRECTX_DIR = 'D:/Programs/DirectX9/include'
STRMBASE_DIR = 'D:/Programs/DirectX9/Samples/C++/DirectShow/BaseClasses'

setup(
    name='vidsrc',
    version=version,
    license='BSD-3-Clause',
    description=description,
    long_description=readme,
    long_description_content_type='text/x-rst',
    author='Christoph Gohlke',
    author_email='cgohlke@cgohlke.com',
    url='https://www.cgohlke.com',
    project_urls={
        'Bug Tracker': 'https://github.com/cgohlke/vidsrc/issues',
        'Source Code': 'https://github.com/cgohlke/vidsrc',
        # 'Documentation': 'https://',
    },
    python_requires='>=3.11',
    install_requires=['numpy'],
    packages=['vidsrc'],
    ext_modules=[
        Extension(
            'vidsrc.vidsrc',
            ['vidsrc/vidsrc.cpp'],
            include_dirs=[DIRECTX_DIR, STRMBASE_DIR, numpy.get_include()],
            library_dirs=[STRMBASE_DIR + '/libraries'],
            libraries=['STRMBASE', 'Ole32', 'OleAut32', 'strmiids'],
        )
    ],
    zip_safe=False,
    platforms=['Windows'],
    classifiers=[
        'Development Status :: 7 - Inactive',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Multimedia :: Video',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: C++',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
    ],
)
